@@ -100,7 +100,7 @@ class log_monitor_manager {
             'log_check_interval' => ['300', 'How often to check logs in seconds (5 minutes)'],
             'ssh_log_file' => ['/var/log/auth.log', 'SSH/Auth log file path'],
             'ftp_log_file' => ['/var/log/proftpd/proftpd.log', 'FTP log file path'],
-            'use_iptables_blocking' => ['0', 'Use iptables for IP blocking (0=software only, 1=iptables)'],
+            'blocking_method' => ['service', 'IP blocking method (service=service-level, iptables=firewall-level)'],
             'docker_network_preserve' => ['1', 'Preserve Docker network access when blocking IPs']
         ];
         
@@ -488,8 +488,12 @@ class log_monitor_manager {
         $block_duration = (int)$this->get_config($block_duration_key, 3600);
         
         $firewall_rule_added = 0;
-        if ($this->get_config('use_iptables_blocking') == '1') {
+        $blocking_method = $this->get_config('blocking_method', 'service');
+        if ($blocking_method === 'iptables') {
             $firewall_rule_added = $this->add_iptables_rule($ip) ? 1 : 0;
+        } else {
+            // Use service-level blocking by default
+            $firewall_rule_added = $this->add_service_blocks($ip) ? 1 : 0;
         }
         
         $stmt = $this->db->prepare('INSERT OR REPLACE INTO service_blocked_ips (service_type, ip_address, block_time, unblock_time, reason, failed_attempts, firewall_rule_added) VALUES (?, ?, ?, ?, ?, ?, ?)');
@@ -509,7 +513,7 @@ class log_monitor_manager {
     }
     
     public function unblock_service_ip($service, $ip) {
-        // Remove iptables rule if it was added
+        // Check what type of blocking was used
         $stmt = $this->db->prepare('SELECT firewall_rule_added FROM service_blocked_ips WHERE service_type = ? AND ip_address = ?');
         $stmt->bindValue(1, $service, SQLITE3_TEXT);
         $stmt->bindValue(2, $ip, SQLITE3_TEXT);
@@ -517,7 +521,12 @@ class log_monitor_manager {
         $row = $result->fetchArray(SQLITE3_ASSOC);
         
         if ($row && $row['firewall_rule_added']) {
-            $this->remove_iptables_rule($ip);
+            $blocking_method = $this->get_config('blocking_method', 'service');
+            if ($blocking_method === 'iptables') {
+                $this->remove_iptables_rule($ip);
+            } else {
+                $this->remove_service_blocks($ip);
+            }
         }
         
         // Remove from service blocked IPs
@@ -535,6 +544,127 @@ class log_monitor_manager {
         return true;
     }
     
+    private function remove_service_blocks($ip) {
+        // Validate IP address first
+        if (!filter_var($ip, FILTER_VALIDATE_IP)) {
+            return false;
+        }
+        
+        $success = true;
+        
+        // Remove SSH service block
+        if (!$this->remove_ssh_service_block($ip)) {
+            $success = false;
+        }
+        
+        // Remove FTP service block
+        if (!$this->remove_ftp_service_block($ip)) {
+            $success = false;
+        }
+        
+        return $success;
+    }
+    
+    private function remove_ssh_service_block($ip) {
+        $ssh_blocked_file = '/etc/ssh/whp_blocked_ips.conf';
+        
+        if (!file_exists($ssh_blocked_file)) {
+            return true; // Nothing to remove
+        }
+        
+        // Read existing blocked IPs
+        $content = file_get_contents($ssh_blocked_file);
+        preg_match_all('/DenyUsers \*@([0-9\.]+)/', $content, $matches);
+        
+        if (!empty($matches[1])) {
+            $blocked_ips = $matches[1];
+            
+            // Remove the IP from the list
+            $blocked_ips = array_filter($blocked_ips, function($blocked_ip) use ($ip) {
+                return $blocked_ip !== $ip;
+            });
+            
+            // Write updated configuration
+            $config_content = "# WHP SSH Blocked IPs - Auto-generated\n";
+            foreach ($blocked_ips as $blocked_ip) {
+                $config_content .= "DenyUsers *@$blocked_ip\n";
+            }
+            
+            if (file_put_contents($ssh_blocked_file, $config_content, LOCK_EX) === false) {
+                return false;
+            }
+            
+            // Set secure file permissions
+            chmod($ssh_blocked_file, 0600);
+            
+            // Reload SSH service gracefully using proc_open for security
+            $descriptors = [1 => ["pipe", "w"], 2 => ["pipe", "w"]];
+            $process = proc_open(["systemctl", "reload", "sshd"], $descriptors, $pipes);
+            if (is_resource($process)) {
+                fclose($pipes[1]);
+                fclose($pipes[2]);
+                $return = proc_close($process);
+                return $return === 0;
+            }
+            return false;
+        }
+        
+        return true;
+    }
+    
+    private function remove_ftp_service_block($ip) {
+        $ftp_blocked_file = '/etc/proftpd/whp_blocked_ips.conf';
+        
+        if (!file_exists($ftp_blocked_file)) {
+            return true; // Nothing to remove
+        }
+        
+        // Read existing blocked IPs
+        $content = file_get_contents($ftp_blocked_file);
+        preg_match_all('/Deny from ([0-9\.]+)/', $content, $matches);
+        
+        if (!empty($matches[1])) {
+            $blocked_ips = $matches[1];
+            
+            // Remove the IP from the list
+            $blocked_ips = array_filter($blocked_ips, function($blocked_ip) use ($ip) {
+                return $blocked_ip !== $ip;
+            });
+            
+            // Write updated configuration
+            $config_content = "# WHP FTP Blocked IPs - Auto-generated\n";
+            $config_content .= "<Directory \"~\">\n";
+            $config_content .= "  <Limit LOGIN>\n";
+            $config_content .= "    Order allow,deny\n";
+            $config_content .= "    Allow from all\n";
+            foreach ($blocked_ips as $blocked_ip) {
+                $config_content .= "    Deny from $blocked_ip\n";
+            }
+            $config_content .= "  </Limit>\n";
+            $config_content .= "</Directory>\n";
+            
+            if (file_put_contents($ftp_blocked_file, $config_content, LOCK_EX) === false) {
+                return false;
+            }
+            
+            // Set secure file permissions
+            chmod($ftp_blocked_file, 0600);
+            
+            // Reload ProFTPD service gracefully using proc_open for security
+            $descriptors = [1 => ["pipe", "w"], 2 => ["pipe", "w"]];
+            $process = proc_open(["systemctl", "reload", "proftpd"], $descriptors, $pipes);
+            if (is_resource($process)) {
+                fclose($pipes[1]);
+                fclose($pipes[2]);
+                $return = proc_close($process);
+                return $return === 0;
+            }
+            return false;
+        }
+        
+        return true;
+    }
+    
     public function is_service_ip_blocked($service, $ip) {
         $stmt = $this->db->prepare('SELECT * FROM service_blocked_ips WHERE service_type = ? AND ip_address = ? AND unblock_time > ?');
         $stmt->bindValue(1, $service, SQLITE3_TEXT);
@@ -546,6 +676,160 @@ class log_monitor_manager {
         return $row ? $row : false;
     }
     
+    private function add_service_blocks($ip) {
+        // Validate IP address first
+        if (!filter_var($ip, FILTER_VALIDATE_IP)) {
+            return false;
+        }
+        
+        $success = true;
+        
+        // Block SSH service
+        if ($this->get_config('ssh_monitoring_enabled') == '1') {
+            if (!$this->add_ssh_service_block($ip)) {
+                $success = false;
+            }
+        }
+        
+        // Block FTP service
+        if ($this->get_config('ftp_monitoring_enabled') == '1') {
+            if (!$this->add_ftp_service_block($ip)) {
+                $success = false;
+            }
+        }
+        
+        return $success;
+    }
+    
+    private function add_ssh_service_block($ip) {
+        $ssh_blocked_file = '/etc/ssh/whp_blocked_ips.conf';
+        
+        // Read existing blocked IPs
+        $blocked_ips = array();
+        if (file_exists($ssh_blocked_file)) {
+            $content = file_get_contents($ssh_blocked_file);
+            preg_match_all('/DenyUsers \*@([0-9\.]+)/', $content, $matches);
+            if (!empty($matches[1])) {
+                $blocked_ips = $matches[1];
+            }
+        }
+        
+        // Add IP if not already blocked
+        if (!in_array($ip, $blocked_ips)) {
+            $blocked_ips[] = $ip;
+            
+            // Write updated configuration
+            $config_content = "# WHP SSH Blocked IPs - Auto-generated\n";
+            foreach ($blocked_ips as $blocked_ip) {
+                $config_content .= "DenyUsers *@$blocked_ip\n";
+            }
+            
+            if (file_put_contents($ssh_blocked_file, $config_content, LOCK_EX) === false) {
+                return false;
+            }
+            
+            // Ensure main sshd_config includes our file
+            $this->ensure_ssh_config_include();
+            
+            // Set secure file permissions
+            chmod($ssh_blocked_file, 0600);
+            
+            // Reload SSH service gracefully using proc_open for security
+            $descriptors = [1 => ["pipe", "w"], 2 => ["pipe", "w"]];
+            $process = proc_open(["systemctl", "reload", "sshd"], $descriptors, $pipes);
+            if (is_resource($process)) {
+                fclose($pipes[1]);
+                fclose($pipes[2]);
+                $return = proc_close($process);
+                return $return === 0;
+            }
+            return false;
+        }
+        
+        return true; // Already blocked
+    }
+    
+    private function add_ftp_service_block($ip) {
+        $ftp_blocked_file = '/etc/proftpd/whp_blocked_ips.conf';
+        
+        // Read existing blocked IPs
+        $blocked_ips = array();
+        if (file_exists($ftp_blocked_file)) {
+            $content = file_get_contents($ftp_blocked_file);
+            preg_match_all('/Deny from ([0-9\.]+)/', $content, $matches);
+            if (!empty($matches[1])) {
+                $blocked_ips = $matches[1];
+            }
+        }
+        
+        // Add IP if not already blocked
+        if (!in_array($ip, $blocked_ips)) {
+            $blocked_ips[] = $ip;
+            
+            // Write updated configuration
+            $config_content = "# WHP FTP Blocked IPs - Auto-generated\n";
+            $config_content .= "<Directory \"~\">\n";
+            $config_content .= "  <Limit LOGIN>\n";
+            $config_content .= "    Order allow,deny\n";
+            $config_content .= "    Allow from all\n";
+            foreach ($blocked_ips as $blocked_ip) {
+                $config_content .= "    Deny from $blocked_ip\n";
+            }
+            $config_content .= "  </Limit>\n";
+            $config_content .= "</Directory>\n";
+            
+            if (file_put_contents($ftp_blocked_file, $config_content, LOCK_EX) === false) {
+                return false;
+            }
+            
+            // Ensure main proftpd.conf includes our file
+            $this->ensure_ftp_config_include();
+            
+            // Set secure file permissions
+            chmod($ftp_blocked_file, 0600);
+            
+            // Reload ProFTPD service gracefully using proc_open for security
+            $descriptors = [1 => ["pipe", "w"], 2 => ["pipe", "w"]];
+            $process = proc_open(["systemctl", "reload", "proftpd"], $descriptors, $pipes);
+            if (is_resource($process)) {
+                fclose($pipes[1]);
+                fclose($pipes[2]);
+                $return = proc_close($process);
+                return $return === 0;
+            }
+            return false;
+        }
+        
+        return true; // Already blocked
+    }
+    
+    private function ensure_ssh_config_include() {
+        $main_config = '/etc/ssh/sshd_config';
+        $include_line = 'Include /etc/ssh/whp_blocked_ips.conf';
+        
+        if (file_exists($main_config)) {
+            $content = file_get_contents($main_config);
+            if (strpos($content, $include_line) === false) {
+                // Add include at the beginning of the file (before other configurations)
+                $content = "# WHP Blocked IPs Include\n$include_line\n\n" . $content;
+                file_put_contents($main_config, $content, LOCK_EX);
+            }
+        }
+    }
+    
+    private function ensure_ftp_config_include() {
+        $main_config = '/etc/proftpd/proftpd.conf';
+        $include_line = 'Include /etc/proftpd/whp_blocked_ips.conf';
+        
+        if (file_exists($main_config)) {
+            $content = file_get_contents($main_config);
+            if (strpos($content, $include_line) === false) {
+                // Add include at the end of the file
+                file_put_contents($main_config, "\n# WHP Blocked IPs Include\n$include_line\n", FILE_APPEND | LOCK_EX);
+            }
+        }
+    }
+    
     private function add_iptables_rule($ip) {
         // Validate IP address first
         if (!filter_var($ip, FILTER_VALIDATE_IP)) {
