@@ -352,18 +352,36 @@ class security_manager {
         $result = $stmt->execute();
         $failed_24h = $result->fetchArray(SQLITE3_ASSOC)['count'];
         
-        // Get whitelist count
+        // Get exemption list count
         $stmt = $this->db->prepare('SELECT COUNT(*) as count FROM whitelist_ips');
         $result = $stmt->execute();
-        $whitelist_count = $result->fetchArray(SQLITE3_ASSOC)['count'];
+        $exemption_count = $result->fetchArray(SQLITE3_ASSOC)['count'];
         
         return array(
             'blocked_ips' => $blocked_count,
             'failed_attempts_24h' => $failed_24h,
-            'whitelist_ips' => $whitelist_count
+            'exemption_ips' => $exemption_count,
+            'whitelist_ips' => $exemption_count // Backward compatibility
         );
     }
     
+    // New exemption terminology methods (for clearer semantics)
+    public function add_exemption_ip($ip_address, $description = '') {
+        return $this->add_whitelist_ip($ip_address, $description);
+    }
+    
+    public function remove_exemption_ip($ip_address) {
+        return $this->remove_whitelist_ip($ip_address);
+    }
+    
+    public function get_exemption_ips() {
+        return $this->get_whitelist_ips();
+    }
+    
+    public function is_ip_exempted($ip_address) {
+        return $this->is_ip_whitelisted($ip_address);
+    }
+    
     public function __destruct() {
         if ($this->db) {
             $this->db->close();
