@@ -327,6 +327,72 @@ class usermgmt {
         }
     }
 
+    /**
+     * Encrypt password using AES-256-GCM if encryption key exists
+     */
+    private function encryptPassword($password) {
+        $keyFile = '/etc/whp/encryption.key';
+        
+        if (!file_exists($keyFile)) {
+            // Fallback to base64 if no encryption key
+            return base64_encode($password);
+        }
+        
+        try {
+            $key = base64_decode(trim(file_get_contents($keyFile)));
+            $iv = random_bytes(12); // 96-bit IV for GCM
+            $encrypted = openssl_encrypt($password, 'aes-256-gcm', $key, OPENSSL_RAW_DATA, $iv, $tag);
+            
+            if ($encrypted === false) {
+                throw new Exception("Encryption failed");
+            }
+            
+            // Combine IV + tag + encrypted data and base64 encode
+            return base64_encode($iv . $tag . $encrypted);
+        } catch (Exception $e) {
+            error_log("Password encryption failed: " . $e->getMessage());
+            throw new Exception("Password encryption failed: " . $e->getMessage());
+        }
+    }
+    
+    /**
+     * Decrypt password using AES-256-GCM if encrypted, otherwise base64 decode
+     */
+    private function decryptPassword($encryptedPassword) {
+        $keyFile = '/etc/whp/encryption.key';
+        
+        if (!file_exists($keyFile)) {
+            // Fallback to base64 decode for legacy passwords
+            return base64_decode($encryptedPassword);
+        }
+        
+        try {
+            $key = base64_decode(trim(file_get_contents($keyFile)));
+            $data = base64_decode($encryptedPassword);
+            
+            // Check if this looks like encrypted data (IV + tag + data = at least 28 bytes)
+            if (strlen($data) < 28) {
+                // Probably legacy base64 encoded password
+                return base64_decode($encryptedPassword);
+            }
+            
+            $iv = substr($data, 0, 12);
+            $tag = substr($data, 12, 16);
+            $encrypted = substr($data, 28);
+            
+            $decrypted = openssl_decrypt($encrypted, 'aes-256-gcm', $key, OPENSSL_RAW_DATA, $iv, $tag);
+            
+            if ($decrypted === false) {
+                throw new Exception("Decryption failed - password may be corrupted");
+            }
+            
+            return $decrypted;
+        } catch (Exception $e) {
+            error_log("Password decryption failed for encrypted data: " . $e->getMessage());
+            throw new Exception("Password decryption failed: " . $e->getMessage());
+        }
+    }
+
     /**
      * Store user password in the database for MySQL authentication
      */
@@ -344,9 +410,8 @@ class usermgmt {
             // Hash the system password for storage
             $password_hash = password_hash($password, PASSWORD_DEFAULT);
             
-            // Encrypt the MySQL password for storage (simple base64 encoding for now)
-            // In production, you'd want to use proper encryption
-            $mysql_password_encrypted = base64_encode($password);
+            // Encrypt the MySQL password for storage
+            $mysql_password_encrypted = $this->encryptPassword($password);
             
             // Insert or update the password record
             $stmt = $pdo->prepare("INSERT INTO whp.user_passwords (username, password_hash, mysql_password) 
@@ -381,8 +446,8 @@ class usermgmt {
             $result = $stmt->fetch();
             
             if ($result) {
-                // Decrypt the MySQL password (base64 decode for now)
-                return base64_decode($result['mysql_password']);
+                // Decrypt the MySQL password
+                return $this->decryptPassword($result['mysql_password']);
             }
             
             return false;
@@ -410,8 +475,8 @@ class usermgmt {
             $result = $stmt->fetch();
             
             if ($result && $result['postgresql_password']) {
-                // Decrypt the PostgreSQL password (base64 decode for now)
-                return base64_decode($result['postgresql_password']);
+                // Decrypt the PostgreSQL password
+                return $this->decryptPassword($result['postgresql_password']);
             }
             
             return false;
@@ -434,8 +499,8 @@ class usermgmt {
                 return false;
             }
             
-            // Encrypt the PostgreSQL password for storage (simple base64 encoding for now)
-            $postgresql_password_encrypted = base64_encode($password);
+            // Encrypt the PostgreSQL password for storage
+            $postgresql_password_encrypted = $this->encryptPassword($password);
             
             // Insert or update the password record
             $stmt = $pdo->prepare("INSERT INTO whp.user_passwords (username, password_hash, mysql_password, postgresql_password) 
