@@ -33,43 +33,55 @@ if ($_SERVER['REQUEST_METHOD'] === 'POST') {
                         $message = 'IP address blocked successfully: ' . $_POST['ip_address'];
                         $message_type = 'success';
                     } else {
-                        $message = 'Error blocking IP address (may be whitelisted)';
+                        $message = 'Error blocking IP address (may be exempted)';
                         $message_type = 'danger';
                     }
                 }
                 break;
-            case 'add_whitelist':
+            case 'add_exemption':
                 if (!empty($_POST['ip_address'])) {
                     $description = !empty($_POST['description']) ? $_POST['description'] : '';
-                    $result = $SecManager->add_whitelist_ip($_POST['ip_address'], $description);
+                    $result = $SecManager->add_exemption_ip($_POST['ip_address'], $description);
                     if ($result) {
-                        $message = 'IP address added to whitelist: ' . $_POST['ip_address'];
+                        $message = 'IP address added to exemption list: ' . $_POST['ip_address'];
                         $message_type = 'success';
                     } else {
-                        $message = 'Error adding IP to whitelist';
+                        $message = 'Error adding IP to exemption list';
                         $message_type = 'danger';
                     }
                 }
                 break;
-            case 'remove_whitelist':
+            case 'remove_exemption':
                 if (!empty($_POST['ip_address'])) {
-                    $result = $SecManager->remove_whitelist_ip($_POST['ip_address']);
+                    $result = $SecManager->remove_exemption_ip($_POST['ip_address']);
                     if ($result) {
-                        $message = 'IP address removed from whitelist: ' . $_POST['ip_address'];
+                        $message = 'IP address removed from exemption list: ' . $_POST['ip_address'];
                         $message_type = 'success';
                     } else {
-                        $message = 'Error removing IP from whitelist';
+                        $message = 'Error removing IP from exemption list';
                         $message_type = 'danger';
                     }
                 }
                 break;
             case 'update_log_monitoring':
+                // Define allowed configuration keys for security
+                $allowed_config_keys = [
+                    'ssh_monitoring_enabled', 'ftp_monitoring_enabled',
+                    'max_ssh_attempts', 'max_ftp_attempts',
+                    'ssh_block_duration', 'ftp_block_duration',
+                    'ssh_log_file', 'ftp_log_file',
+                    'log_check_interval', 'blocking_method',
+                    'docker_network_preserve'
+                ];
+                
                 $config_updates = array();
                 
                 foreach ($_POST as $key => $value) {
                     if (strpos($key, 'config_') === 0) {
                         $setting_name = substr($key, 7); // Remove 'config_' prefix
-                        $config_updates[$setting_name] = $value;
+                        if (in_array($setting_name, $allowed_config_keys)) {
+                            $config_updates[$setting_name] = $value;
+                        }
                     }
                 }
                 
@@ -106,7 +118,7 @@ if ($_SERVER['REQUEST_METHOD'] === 'POST') {
 
 // Get data
 $blocked_ips = $SecManager->get_blocked_ips();
-$whitelist_ips = $SecManager->get_whitelist_ips();
+$exemption_ips = $SecManager->get_exemption_ips();
 $security_stats = $SecManager->get_security_stats();
 $recent_logins = $SecManager->get_all_recent_logins(10);
 
@@ -159,11 +171,11 @@ $access_stats = $PermManager->get_system_access_stats(30);
     <div class="col-md-4">
         <div class="card h-100">
             <div class="card-header">
-                <h5 class="card-title mb-0">Whitelisted IPs</h5>
+                <h5 class="card-title mb-0">Exempted IPs</h5>
             </div>
             <div class="card-body text-center">
-                <h2 class="text-success"><?php echo $security_stats['whitelist_ips']; ?></h2>
-                <p class="mb-0">IP addresses in whitelist</p>
+                <h2 class="text-success"><?php echo $security_stats['exemption_ips']; ?></h2>
+                <p class="mb-0">IP addresses in exemption list</p>
             </div>
         </div>
     </div>
@@ -402,12 +414,21 @@ $access_stats = $PermManager->get_system_access_stats(30);
                             <h6 class="mb-3">Advanced Options</h6>
                             
                             <div class="mb-3">
-                                <div class="form-check">
-                                    <input class="form-check-input" type="checkbox" name="config_use_iptables_blocking" value="1"
-                                           <?php echo ($LogMonitor->get_config('use_iptables_blocking') == '1') ? 'checked' : ''; ?>>
-                                    <label class="form-check-label">Use iptables for IP blocking</label>
-                                    <small class="form-text text-muted">When enabled, uses firewall rules in addition to application-level blocking</small>
-                                </div>
+                                <label class="form-label">IP Blocking Method</label>
+                                <select class="form-select" name="config_blocking_method">
+                                    <?php 
+                                    $blocking_method = $LogMonitor->get_config('blocking_method', 'service');
+                                    $methods = [
+                                        'service' => 'Service-Level Blocking (Recommended)',
+                                        'iptables' => 'Firewall-Level Blocking (iptables)'
+                                    ];
+                                    foreach ($methods as $value => $label): ?>
+                                    <option value="<?php echo $value; ?>" <?php echo ($blocking_method == $value) ? 'selected' : ''; ?>>
+                                        <?php echo $label; ?>
+                                    </option>
+                                    <?php endforeach; ?>
+                                </select>
+                                <small class="form-text text-muted">Service-level blocking uses configuration files and graceful service reloads (safer for Docker)</small>
                             </div>
                             
                             <div class="mb-3">
@@ -415,7 +436,7 @@ $access_stats = $PermManager->get_system_access_stats(30);
                                     <input class="form-check-input" type="checkbox" name="config_docker_network_preserve" value="1"
                                            <?php echo ($LogMonitor->get_config('docker_network_preserve') == '1') ? 'checked' : ''; ?>>
                                     <label class="form-check-label">Preserve Docker networking</label>
-                                    <small class="form-text text-muted">Block only specific services instead of all traffic from IP</small>
+                                    <small class="form-text text-muted">When using iptables: block only specific services instead of all traffic from IP</small>
                                 </div>
                             </div>
                         </div>
@@ -452,25 +473,25 @@ $access_stats = $PermManager->get_system_access_stats(30);
             </div>
         </div>
     </div>
-    <!-- Add to Whitelist Panel -->
+    <!-- Add to Exemption List Panel -->
     <div class="col-md-6">
         <div class="card h-100">
             <div class="card-header">
-                <h5 class="card-title mb-0">Add to Whitelist</h5>
+                <h5 class="card-title mb-0">Add to Exemption List</h5>
             </div>
             <div class="card-body">
                 <form method="POST" action="">
                     <input type="hidden" name="csrf_token" value="<?php echo get_csrf_token(); ?>">
-                    <input type="hidden" name="action" value="add_whitelist">
+                    <input type="hidden" name="action" value="add_exemption">
                     <div class="mb-3">
-                        <label for="whitelist_ip" class="form-label">IP Address</label>
-                        <input type="text" class="form-control" id="whitelist_ip" name="ip_address" placeholder="192.168.1.100" required>
+                        <label for="exemption_ip" class="form-label">IP Address</label>
+                        <input type="text" class="form-control" id="exemption_ip" name="ip_address" placeholder="192.168.1.100" required>
                     </div>
                     <div class="mb-3">
-                        <label for="whitelist_description" class="form-label">Description (optional)</label>
-                        <input type="text" class="form-control" id="whitelist_description" name="description" placeholder="Office, Home, etc.">
+                        <label for="exemption_description" class="form-label">Description (optional)</label>
+                        <input type="text" class="form-control" id="exemption_description" name="description" placeholder="Office, Home, etc.">
                     </div>
-                    <button type="submit" class="btn btn-success">Add to Whitelist</button>
+                    <button type="submit" class="btn btn-success">Add to Exemption List</button>
                 </form>
             </div>
         </div>
@@ -509,11 +530,11 @@ $access_stats = $PermManager->get_system_access_stats(30);
             </div>
         </div>
     </div>
-    <!-- Whitelist Table -->
+    <!-- Exemption List Table -->
     <div class="col-md-6">
         <div class="card h-100">
             <div class="card-header">
-                <h5 class="card-title mb-0">Whitelisted IPs</h5>
+                <h5 class="card-title mb-0">Exempted IPs</h5>
             </div>
             <div class="card-body">
                 <div class="table-responsive">
@@ -522,14 +543,14 @@ $access_stats = $PermManager->get_system_access_stats(30);
                             <tr><th>IP Address</th><th>Description</th><th width="120">Action</th></tr>
                         </thead>
                         <tbody>
-                            <?php foreach ($whitelist_ips as $ip): ?>
+                            <?php foreach ($exemption_ips as $ip): ?>
                             <tr>
                                 <td><?php echo htmlspecialchars($ip['ip_address']); ?></td>
                                 <td><?php echo htmlspecialchars($ip['description']); ?></td>
                                 <td>
                                     <form method="POST" action="" style="display:inline;">
                                         <input type="hidden" name="csrf_token" value="<?php echo get_csrf_token(); ?>">
-                                        <input type="hidden" name="action" value="remove_whitelist">
+                                        <input type="hidden" name="action" value="remove_exemption">
                                         <input type="hidden" name="ip_address" value="<?php echo htmlspecialchars($ip['ip_address']); ?>">
                                         <button type="submit" class="btn btn-danger btn-sm">Remove</button>
                                     </form>
